'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _stateDoc = require('./stateDoc');

var _stateDoc2 = _interopRequireDefault(_stateDoc);

var _vm = require('vm');

var _vm2 = _interopRequireDefault(_vm);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var fs = require('fs');
var path = require('path');
var getRequires = require('./getRequires');

var _require = require('./parseModule'),
    parseModule = _require.parseModule;

function clone(obj) {
	if (null == obj || 'object' != (typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj))) return obj;
	var copy = obj.constructor();
	for (var attr in obj) {
		if (obj.hasOwnProperty(attr)) copy[attr] = obj[attr];
	}
	return copy;
}

function getMixins(code, file) {
	try {
		var requiresFromComponent = getRequires(code);
		var output = [];
		(0, _keys2.default)(requiresFromComponent).forEach(function (reqFromComponent) {
			var tempRequire = reqFromComponent.split('/');
			if (tempRequire[0] === '.' || tempRequire[0] === '..') {
				var folderFile = path.dirname(file);
				var pathRequire = path.join(path.normalize(folderFile), reqFromComponent) + '.js';
				if (fs.existsSync(pathRequire)) {
					var source = fs.readFileSync(pathRequire, {
						encoding: 'utf-8'
					});
					_stateDoc2.default.saveMixin(source, pathRequire);
					if (_stateDoc2.default.isMixin()) {
						var parsedSource = parseModule(source, _stateDoc2.default.jscodeLang);
						var mixin = evalComponentCode(parsedSource);
						if ((0, _keys2.default)(mixin.exports).length === 0) {
							mixin.exports.default = mixin.module.exports;
						}
						if (mixin.exports.default) {
							output.push(mixin.exports.default);
						}
					}
				}
			}
		});
		return output;
	} catch (err) {
		throw err;
	}
}

var evalComponentCode = function evalComponentCode(code) {
	try {
		var script = new _vm2.default.Script(code, {});
		var requireSanbox = function requireSanbox(element) {
			if (element === 'vuex') {
				return {
					mapState: function mapState() {},
					mapMutations: function mapMutations() {},
					mapGetters: function mapGetters() {},
					mapActions: function mapActions() {}
				};
			}
			if (element === 'vue') {
				return {
					__esModule: true,
					use: function use() {},
					component: function component() {},
					extended: function extended() {},
					default: {
						extend: function extend(obj) {
							return obj;
						}
					}
				};
			}
			return function () {};
		};
		requireSanbox.context = function () {
			return function () {};
		};
		var sandbox = {
			exports: {},
			module: {
				exports: {}
			},
			require: requireSanbox,
			document: {},
			window: {
				location: {}
			},
			alert: function alert() {},
			confirm: function confirm() {},

			console: {
				log: function log() {},
				debug: function debug() {}
			},
			sessionStorage: {
				getItem: function getItem() {},
				setItem: function setItem() {},
				removeItem: function removeItem() {}
			},
			localStorage: {
				getItem: function getItem() {},
				setItem: function setItem() {},
				removeItem: function removeItem() {}
			}
		};
		var context = new _vm2.default.createContext(sandbox);
		script.runInContext(context);
		var output = sandbox;
		return clone(output);
	} catch (err) {
		throw err;
	}
};

module.exports = function getSandbox(stateDoc, file) {
	var parsedSource = parseModule(stateDoc.jscodeReqest, stateDoc.jscodeLang);
	var component = evalComponentCode(parsedSource).exports;
	var mixins = getMixins(parsedSource, file).reverse();
	component.default.mixins = mixins;
	return component;
};